package run.evan.dd;

import android.content.Context;
import android.location.Location;
import android.net.wifi.ScanResult;
import android.os.Parcel;
import android.telephony.CellInfo;
import android.telephony.CellInfoCdma;
import android.telephony.CellInfoGsm;
import android.telephony.CellInfoLte;
import android.telephony.CellInfoNr;
import android.telephony.CellInfoTdscdma;
import android.telephony.CellInfoWcdma;
import android.widget.Toast;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class DdHelper {

    static Random rnd = new Random();


    static {
        rnd.setSeed(System.currentTimeMillis());
    }


    public static int getScale(double val){
        BigDecimal bigDecimal = new BigDecimal(String.valueOf(val));
        return bigDecimal.scale();
    }

    public static double parseDouble(String str){
        return Double.parseDouble(str) ;
    }

    public static double randLatLon(String str) {
        BigDecimal b = new BigDecimal(str);
        BigDecimal add = b.add(new BigDecimal(String.valueOf("0.000011121")).multiply(BigDecimal.valueOf(rnd.nextInt(10) + 1)));
        return Double.parseDouble(add.toString()) ;
    }

    public static void saveCellInfo(List<CellInfo> cellInfos, String path) {
        String s = convertCellInfo(cellInfos);
        writeFile(s.getBytes(), path);
    }

    public static List<CellInfo> readCellInfo(String path) {
        List<CellInfo> cellInfos = parseCellInfo(new String(readFile(path)));
        return cellInfos;
    }


    public static void saveScanResult(List<ScanResult> scanResults, String path) {
        String s = convertScanResult(scanResults);
        writeFile(s.getBytes(), path);
    }

    public static List<ScanResult> readScanResult(String path) {
        List<ScanResult> scanResults = parseScanResult(new String(readFile(path)));
        return scanResults;
    }

    public static String convertCellInfo(List<CellInfo> cellInfos) {
        List<Tuple> outData = new ArrayList<>();
        for (CellInfo cellInfo : cellInfos) {
            Tuple t = new Tuple();
            Parcel parcel = Parcel.obtain();
            cellInfo.writeToParcel(parcel, 0);
            byte[] marshall = parcel.marshall();
            parcel.recycle();
            t.setClassName(cellInfo.getClass().getName());
            t.setData(marshall);
            outData.add(t);
        }
        return JSON.toJSONString(outData);
    }

    public static List<CellInfo> parseCellInfo(String json) {
        List<CellInfo> result = new ArrayList<>();
        List<Tuple> tuples = JSON.parseObject(json, new TypeReference<List<Tuple>>() {
        });
        for (Tuple tuple : tuples) {
            Parcel p = Parcel.obtain();
            p.unmarshall(tuple.getData(), 0, tuple.getData().length);
            p.setDataPosition(0);

            switch (tuple.getClassName()) {
                case "android.telephony.CellInfoNr":
                    result.add(CellInfoNr.CREATOR.createFromParcel(p));
                    break;
                case "android.telephony.CellInfoLte":
                    result.add(CellInfoLte.CREATOR.createFromParcel(p));
                    break;
                case "android.telephony.CellInfoWcdma":
                    result.add(CellInfoWcdma.CREATOR.createFromParcel(p));
                    break;
                case "android.telephony.CellInfoCdma":
                    result.add(CellInfoCdma.CREATOR.createFromParcel(p));
                    break;
                case "android.telephony.CellInfoTdscdma":
                    result.add(CellInfoTdscdma.CREATOR.createFromParcel(p));
                    break;
                case "android.telephony.CellInfoGsm":
                    result.add(CellInfoGsm.CREATOR.createFromParcel(p));
                    break;
            }
            p.recycle();

        }
        return result;
    }

    public static String convertScanResult(List<ScanResult> scanResults) {
        List<Tuple> outData = new ArrayList<>();
        for (ScanResult scanResult : scanResults) {
            Tuple t = new Tuple();
            Parcel parcel = Parcel.obtain();
            scanResult.writeToParcel(parcel, 0);
            byte[] marshall = parcel.marshall();
            parcel.recycle();
            t.setClassName(scanResult.getClass().getName());
            t.setData(marshall);
            outData.add(t);
        }
        return JSON.toJSONString(outData);
    }

    public static List<ScanResult> parseScanResult(String json) {
        List<ScanResult> result = new ArrayList<>();
        List<Tuple> tuples = JSON.parseObject(json, new TypeReference<List<Tuple>>() {
        });
        for (Tuple tuple : tuples) {
            Parcel p = Parcel.obtain();
            p.unmarshall(tuple.getData(), 0, tuple.getData().length);
            p.setDataPosition(0);
            result.add(ScanResult.CREATOR.createFromParcel(p));
            p.recycle();
        }
        return result;
    }


    public static void writeFile(byte[] data, String filePath) {
        try (FileOutputStream fileOutputStream = new FileOutputStream(filePath)) {
            fileOutputStream.write(data);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void writeFileString(String str, String filePath) {
        writeFile(str.getBytes(), filePath);
    }

    public static byte[] readFile(String filePath) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try (FileInputStream fileInputStream = new FileInputStream(filePath)) {
            byte[] b = new byte[1024];
            int n;
            while ((n = fileInputStream.read(b)) != -1) {
                out.write(b, 0, n);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return out.toByteArray();
    }

    public static String readFileString(String filePath) {
        return new String(readFile(filePath));
    }

    public static void isDirExist(String str) {
        File file = new File(str);
        if (!file.exists()) {
            file.mkdirs();
        }
    }

    public static void toast(Context ctx, String str) {
        Toast.makeText(ctx, str, Toast.LENGTH_SHORT).show();
    }

}
